#include "billes.h"

nzGame::nzGame() {
  symbols[0] = "X";
  symbols[1] = "[1m[33mO[0m";  // jaune
  symbols[2] = "[1m[34mO[0m";  // bleu

  reset();
  cout << endl;
  display();
}

bool nzGame::tryMove(int moveTo) {
  if (allowed(moveTo)) {
    moveHole(moveTo);
    display();
    return true;
  } else {
    return false;
  }
}

bool nzGame::allowed(int moveTo) {
  // regles du jeu
  int d = moveTo - holePos;
  int absD = abs(d);
  string couleur = cases[moveTo]==1 ? "jaune" : "bleu";

  // indice valable ?
  if (moveTo < 0 || moveTo > 16) {
    cout << "hors plateau" << endl;
    return false;
  } else {
    // cout << "case " << moveTo << " ";
    // accessible ?
    if (!accessible(holePos, moveTo)) {
      cout << "inaccessible, ";
      return false;
    } else {
      // dans le bon sens ?
      if (sgn(d) != sgn(2*cases[moveTo]-3)) {
	// cout << "recule, ";
	return false;
      } else {
	// shift ?
	if (absD == 1 || absD == 3) {
	  // cout << "shift " << couleur << ", ";
	  return true;
	  // jump ?
	} else if (absD == 2 || absD == 6) {
	  // jump bonne couleur ?
	  if (cases[(moveTo+holePos)/2] == cases[moveTo]) {
	    // cout << "jump meme couleur, ";
	    return false;
	  } else {
	    // cout << "jump " << couleur << ", ";
	    return true;
	  }
	  // autres
	} else {
	  cout << "mvmt impossible" << endl;
	  return false;
	}
      }
    }
  }
}

bool nzGame::accessible(int from, int to) {
  bool acces = false;
  for (int i=0; i<8; i++) {
    if (voisins[from][i] == to) acces = true;
  }
  return acces;
}

void nzGame::playSequence(vector<int> seq) {
  for (int k=0; k<seq.size(); k++) {
    if (allowed(seq[k])) {
      moveHole(seq[k]);
      display();
    }
  }
}

int nzGame::score() {
  // compte le nombre de billes arrivees de l'autre cote
  int s=0;
  for (int k=0; k<8; k++) {
    if (cases[k] == 2) s++;
  }
  for (int k=9; k<17; k++) {
    if (cases[k] == 1) s++;
  }
  return s;
}

void nzGame::moveHole(int moveTo) {
  cases[holePos] = cases[moveTo];
  cases[moveTo] = 0;
  holePos = moveTo;
  coups.push_back(moveTo);
  int leScore = score();
  if (leScore == 16) gameWon();
}

void nzGame::reset() {
  // remise a zero du plateau
  cout << "Resetting board..." << endl;
  for (int i=0; i<8; i++) {
    cases[i] = 1;
  }
  cases[8] = 0;
  for (int i=9; i<17; i++) {
    cases[i] = 2;
  }
  
  holePos = 8;
  coups.clear();
  coups.push_back(8);
}

int nzGame::lastMove() {
  // renvoie le dernier coup joue
  return coups.back();
}

void nzGame::resetTo(vector<int> seq) {
  // remise dans un etat determine
  reset();
  for (int k=0; k<seq.size(); k++) {
    if (allowed(seq[k])) {
      moveHole(seq[k]);
    }
  }
}

void nzGame::display() {
  // affiche le plateau dans le shell
  system("clear");
  cout << endl;
  cout << "       " << symbols[cases[2]] << "       " << symbols[cases[10]] << endl;
  cout << "  ";
  for (int k=1; k<17; k+=4) { cout << "   " << symbols[cases[k]]; }
  cout << endl;
  for (int k=0; k<17; k+=4) { cout << "   " << symbols[cases[k]]; }
  cout << endl << "  ";
  for (int k=3; k<17; k+=4) { cout << "   " << symbols[cases[k]]; }
  cout << endl;
  cout << "       " << symbols[cases[6]] << "       " << symbols[cases[14]] << endl << endl;
  displayCoups();
  cout << "score " << score() << endl;
}

void nzGame::displayCoups() {
  cout << "coups : ";
  for (int i=0; i<coups.size(); i++) {
    cout << coups[i] << "-";
  }
  cout << endl;
}

void nzGame::writeSeq(int tentative, int m) {
  ofstream fout("sequences", ios::app);
  fout << "score " << score() << " coups "; 
  for (int i=0; i<coups.size(); i++) {
    fout << coups[i] << "-";
  }
  fout << "(" << tentative << "?) m=" << m << endl;
  fout.close();
}

void nzGame::writeSeq() {
  ofstream fout("sequences", ios::app);
  fout << "score " << score() << " coups "; 
  for (int i=0; i<coups.size(); i++) {
    fout << coups[i] << "-";
  }
  fout << "(" << coups.size() << " coups)" << endl;
  fout.close();
}

void nzGame::gameWon() {
  ofstream fout("solutions", ios::app);
  for (int i=0; i<coups.size(); i++) {
    fout << coups[i] << "-";
  }
  fout << "(" << coups.size() << " coups)" << endl;
  fout.close();
  cout << "Victoire !" << endl;
}

int nzGame::sgn(int a) { return (a>0)?1:-1; }



